@@ -1,17 +1,16 @@
-from contextlib import contextmanager
-from typing import Type, List
+from typing import Type, List, Callable
 
 from PIL import Image
+from bitfield import BitField
 from django.apps import apps
 from django.contrib.contenttypes.fields import GenericForeignKey
 from django.contrib.contenttypes.models import ContentType
-from django.core.exceptions import ValidationError
 from django.db import models
 from django.db.models.base import ModelBase
-from django.db.models.fields.files import ImageFieldFile
+from django.db.models.fields.files import FieldFile
 from django.utils.translation import gettext_lazy as _
 
-from image_assets import defaults
+from image_assets import defaults, validators
 
 
 class AssetTypeManager(models.Manager):
@@ -44,8 +43,8 @@ class AssetType(models.Model):
     )
 
     slug = models.SlugField(verbose_name=_("Slug"), unique=True)
-    format = models.CharField(
-        verbose_name=_('Image Format'), max_length=4, choices=FORMAT_CHOICES)
+    formats = BitField(verbose_name=_('Formats'), flags=FORMAT_CHOICES,
+                       default=0)
     min_width = models.IntegerField(
         verbose_name=_('Min Width'), default=0)
     min_height = models.IntegerField(
@@ -76,67 +75,73 @@ class AssetType(models.Model):
     def __str__(self):
         return self.slug
 
-    @classmethod
-    def validate_asset(cls, value: ImageFieldFile):
-        asset = value.instance
-        if asset.asset_type_id is None:
-            # asset type not filled, no data to validate
-            return
-        errors = []
-        asset_type: AssetType = asset.asset_type
+    # noinspection PyUnusedLocal
+    def get_validators(self, file: Image.Image
+                       ) -> List[Callable[[Image.Image], List[str]]]:
+        """
+        Returns list of checks to run against file.
 
-        # validate file size
-        if (asset_type.max_size and value.size and
-                asset_type.max_size < value.size):
+        :param file: opened image file.
+        :return: list of check methods.
+        """
+        return [
+            self.validate_format,
+            self.validate_dimensions,
+            self.validate_aspect,
+        ]
+
+    def validate_max_size(self, value: FieldFile) -> List[str]:
+        """ Validate max file size."""
+        if (self.max_size and value.size and
+                self.max_size < value.size):
             msg = _('File size must be not greater than %s')
-            errors.append(msg % asset_type.max_size)
+            return [msg % self.max_size]
+        return []
 
-        # open image and validate it's content
-        with cls.open_file(value.file) as file:
-            validation_errors = cls.validate_file(file, asset_type)
-            errors.extend(validation_errors)
+    def validate_format(self, file: Image.Image) -> List[str]:
+        """ Validate allowed format list."""
+        fmt = file.format.lower()
+        set_flags = dict(self.formats.items())
+        if self.formats and not set_flags.get(fmt):
+            msg = _('Image format must be one of %s')
+            formats = ','.join([self.formats.get_label(k)
+                                for k, v in self.formats.items() if v])
+            return [msg % formats]
+        return []
 
-        if errors:
-            raise ValidationError(errors)
-
-    @classmethod
-    @contextmanager
-    def open_file(cls, file):
-        with Image.open(file) as file_content:
-            yield file_content
-
-    @classmethod
-    def validate_file(cls, file: Image.Image, asset_type) -> List:
+    def validate_dimensions(self, file: Image.Image) -> List[str]:
+        """ Validate minimum image width and height."""
         errors = []
-        # internal image format
-        if file.format.lower() != asset_type.format:
-            msg = _('Image format must be %s')
-            errors.append(msg % asset_type.format)
         # image width
-        if file.width and asset_type.min_width > file.width:
+        if file.width and self.min_width > file.width:
             msg = _('Image width must be not less than %s')
-            errors.append(msg % asset_type.min_width)
+            errors.append(msg % self.min_width)
         # image height
-        if file.height and asset_type.min_height > file.height:
+        if file.height and self.min_height > file.height:
             msg = _('Image height must be not less than %s')
-            errors.append(msg % asset_type.min_height)
-        if file.width and file.height and asset_type.aspect:
-            image_aspect = file.width / file.height
-            delta = abs(image_aspect - asset_type.aspect)
-            if asset_type.accuracy == 0:
-                if image_aspect != asset_type.aspect:
-                    msg = _('Image aspect must be %s')
-                    errors.append(msg % asset_type.aspect)
-            elif round(delta / asset_type.accuracy) > 1:
-                # round at scale of accuracy
-                msg = _('Image aspect must be %(aspect)s ± %(accuracy)s')
-                args = {
-                    'aspect': asset_type.aspect,
-                    'accuracy': asset_type.accuracy
-                }
-                errors.append(msg % args)
+            errors.append(msg % self.min_height)
         return errors
 
+    def validate_aspect(self, file: Image.Image) -> List[str]:
+        """ Validate image aspect ratio with accuracy."""
+        if not (file.width and file.height and self.aspect):
+            return []
+        image_aspect = file.width / file.height
+        delta = abs(image_aspect - self.aspect)
+        if self.accuracy == 0:
+            if image_aspect != self.aspect:
+                msg = _('Image aspect must be %s')
+                return [msg % self.aspect]
+        elif round(delta / self.accuracy) > 1:
+            # round at scale of accuracy
+            msg = _('Image aspect must be %(aspect)s ± %(accuracy)s')
+            args = {
+                'aspect': self.aspect,
+                'accuracy': self.accuracy
+            }
+            return [msg % args]
+        return []
+
 
 def get_asset_type_model() -> Type[AssetType]:
     app_label, model_name = defaults.ASSET_TYPE_MODEL.split('.')
@@ -145,7 +150,7 @@ def get_asset_type_model() -> Type[AssetType]:
 
 class Asset(models.Model):
     image = models.ImageField(
-        verbose_name=_('Image'), validators=[AssetType.validate_asset])
+        verbose_name=_('Image'), validators=[validators.AssetValidator()])
     asset_type = models.ForeignKey(
         AssetType, models.CASCADE, verbose_name=_('Asset Type'))
     active = models.BooleanField(verbose_name=_('Active'), default=True)
@@ -211,7 +216,7 @@ class DeletedAsset(models.Model):
             object_id=self.object_id,
             image=self.image,
             active=False)
-        qs = DeletedAsset.objects.filter(pk=self.pk)
+        qs = get_deleted_asset_model().objects.filter(pk=self.pk)
         # skip sending pre_delete/post_delete signals to prevent file removal.
         # noinspection PyProtectedMember
         qs._raw_delete(qs.db)
